/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.coldchain.modular.monitornotice.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.coldchain.core.alarm.bean.SensorAlarm;
import vip.xiaonuo.coldchain.core.alarm.service.SensorAlarmService;
import vip.xiaonuo.coldchain.modular.app.param.export.AppTrendParam;
import vip.xiaonuo.coldchain.modular.app.service.AppDeviceService;
import vip.xiaonuo.coldchain.modular.monitordevice.entity.MonitorDevice;
import vip.xiaonuo.coldchain.modular.monitordevice.service.MonitorDeviceService;
import vip.xiaonuo.coldchain.modular.monitornotice.entity.CountWithTime;
import vip.xiaonuo.coldchain.modular.monitornotice.entity.Realtime;
import vip.xiaonuo.coldchain.modular.monitornotice.entity.Trend;
import vip.xiaonuo.coldchain.modular.monitornotice.param.MonitorNoticePageParam;
import vip.xiaonuo.coldchain.modular.monitornotice.param.RealtimeParam;
import vip.xiaonuo.coldchain.modular.monitornotice.param.TrendParam;
import vip.xiaonuo.coldchain.modular.monitornotice.service.MonitorNoticeService;
import vip.xiaonuo.common.pojo.CommonResult;

import java.util.List;

/**
 * 监控通知控制器
 *
 * @author 黄渊昊
 * @date 2024/11/25 10:58
 */
@Tag(name = "监控通知控制器")
@RestController
@Validated
public class MonitorNoticeController {

    @Resource
    private MonitorNoticeService monitorNoticeService;
    @Resource
    private SensorAlarmService sensorAlarmService;
    @Resource
    private MonitorDeviceService monitorDeviceService;
    @Resource
    private AppDeviceService appDeviceService;

    /**
     * 获取监控通知分页
     *
     * @author 黄渊昊
     * @date 2024/11/25 10:58
     */
    @Operation(summary = "获取监控通知分页")
    @SaCheckPermission("/coldchain/monitornotice/page")
    @GetMapping("/coldchain/monitornotice/page")
    public CommonResult<Page<SensorAlarm>> page(MonitorNoticePageParam monitorNoticePageParam) {
        return CommonResult.data(sensorAlarmService.getMessages(monitorNoticePageParam));
    }

    /**
     * 获取告警类型
     */
    @Operation(summary = "获取告警类型")
    @SaCheckPermission("/coldchain/monitornotice/getAlarmTypeList")
    @GetMapping("/coldchain/monitornotice/getAlarmTypeList")
    public CommonResult<List<String>> getAlarmTypeList(String type) {
        return CommonResult.data(sensorAlarmService.getAlarmTypeList(type));
    }

    /**
     * 获取监控通知列表
     *
     * @author 黄渊昊
     */
    @Operation(summary = "获取监控通知列表")
    @SaCheckPermission("/coldchain/monitornotice/list")
    @GetMapping("/coldchain/monitornotice/list")
    public CommonResult<List<SensorAlarm>> list() {
        return CommonResult.data(sensorAlarmService.list());
    }

    /**
     * 获取近三天报警情况
     */
    @Operation(summary = "获取近三天报警情况")
    @SaCheckPermission("/coldchain/monitornotice/getNoticeByDate")
    @GetMapping("/coldchain/monitornotice/getNoticeByDate")
    public CommonResult<List<CountWithTime>> getNoticeByDate() {
        return CommonResult.data(monitorNoticeService.getNoticeByDate());
    }

    /**
     * 获取冰箱点位监控数据趋势
     */
    @Operation(summary = "获取冰箱点位监控数据趋势")
    @SaCheckPermission("/coldchain/monitornotice/getTrend")
    @GetMapping("/coldchain/monitornotice/getTrend")
    public CommonResult<Trend> getTrendList(TrendParam trendParam) {
        return CommonResult.data(monitorNoticeService.getTrend(trendParam));
    }

    /**
     * 获取冰箱实时数据
     */
    @Operation(summary = "获取冰箱实时数据")
    @SaCheckPermission("/coldchain/monitornotice/getRealtime")
    @GetMapping("/coldchain/monitornotice/getRealtime")
    public CommonResult<Realtime> getRealtime(RealtimeParam realtimeParam) {
        return CommonResult.data(monitorNoticeService.getRealtime(realtimeParam));
    }


    /**
     * 导出传感器数据
     */
    @Operation(summary = "导出传感器数据")
    @GetMapping("/coldchain/monitornotice/export")
    public void export(HttpServletResponse response, TrendParam trendParam) {
        MonitorDevice monitorDevice = monitorDeviceService.queryEntity(trendParam.getDeviceId());
        AppTrendParam appTrendParam = BeanUtil.copyProperties(trendParam, AppTrendParam.class);
        appTrendParam.setSensorCode(monitorDevice.getDeviceCode());
        appDeviceService.export(response, appTrendParam);
    }
}
